from qt import *
from kdeemul import *

from importsession import LDirImport 
from sanetools import LSaneImage
import threading
import commands
import paths
#from organizeview import LThumbnail
#from utils import * 

class LScannerPanel(QWidget):
	def __init__(self, parent):
		QWidget.__init__(self, parent)
		self.stack = parent
		self.sane = None
		self.scanner_image = QPixmap(paths.art("scanner.png"))
		self.progress_button = QPixmap(paths.art("progress.png"))
		hbox = QHBoxLayout(self,0,6,"Main Layout")
		
		hbox.insertStretch(-1)

		self.scannerButton = QPushButton("Scanner", self)
		self.scannerButton.setPixmap(self.scanner_image)
		QToolTip.add(self.scannerButton, i18n("Scan Image")) 
		hbox.addWidget(self.scannerButton)
		
		self.state = "ready"

		self.container = QWidget(self,"Container")
		self.container.setMinimumWidth(500)
		hbox.addWidget(self.container)
		vbox = QVBoxLayout(self.container,30,6,"Container Layout")

		self.scannerLabel = QLabel(i18n("Initializing..."), self.container)
		self.scannerLabel.setFrameStyle(QFrame.Panel)
		self.scannerLabel.setFrameShadow(QFrame.Plain)
		self.scannerLabel.setLineWidth(1)
		vbox.addWidget(self.scannerLabel)
		self.scannerLabel.setAlignment(Qt.AlignHCenter and Qt.AlignVCenter)

		self.button_container = QWidget(self.container,"Button Container")
		vbox.addWidget(self.button_container)
		self.button_hbox = QHBoxLayout(self.button_container,0,6,"Container Layout")

		self.rescan_button = QPushButton(self.button_container,"Rescan Button")
		self.rescan_button.setText(i18n("Scanner Check"))
		QToolTip.add(self.rescan_button, i18n("Check system for available scanners")) 
		self.button_hbox.addWidget(self.rescan_button)

		self.button_hbox.insertSpacing(-1,10)

		self.resolution_setting = KComboBox(self.button_container, "Resolution Box")
		self.resolution_setting.insertItem("Resolution: 75",0)
		self.resolution_setting.insertItem("Resolution: 150",1)
		self.resolution_setting.insertItem("Resolution: 300",2)
		self.button_hbox.addWidget(self.resolution_setting)

		hbox.insertStretch(-1)
		
		self.timer = QTimer(self,"Timer")
		self.progress = 0
		
		self.initScanner()
		self.initSignals()
		
	def initScanner(self):
		if (self.sane and self.sane.scanner):
			self.sane.scanner.close()
			self.sane = None
		self.sane = LSaneImage()
		test = self.sane.getFirstScanner()
		if (test):
			self.scannerLabel.setText("Ready...")
		else:
			self.scannerButton.setEnabled(False)	
			self.scannerLabel.setText("Unable to find scanner...")
			self.state = "disconnected"
			
	def initSignals(self):
		QObject.connect(self.scannerButton, SIGNAL("clicked()"), self.startScanning)
		QObject.connect(self.rescan_button, SIGNAL("clicked()"), self.initScanner)
		QObject.connect(self.timer, SIGNAL("timeout()"), self.getImage)
		
	def slotShowPanel(self):
		pass
		
	def startScanning(self):
		self.scannerButton.setPixmap(self.progress_button)
		self.scannerLabel.setText("Scanning...")
		self.state = "scanning"
		self.timer.start(1000)
		self.thread = threading.Thread(target=self.scanImage)
		self.thread.start()

	def scanImage(self):
		data = str(self.resolution_setting.currentText())
		label, res = data.split(" "); res = int(res)
		print res
		self.sane.scanImage(res)
		self.state = "complete" 

	def getImage(self):
		if (self.state == "complete"):
			self.progress = 0
			files = []
			self.timer.stop()
			image = self.sane.image
			self.sane.image = None
			filename = commands.getoutput("tempfile -p IMG- -s .jpg")
			if (image):
				image.save(filename)
				files.append(filename)
				images = LDirImport(files)
				importPanel = self.stack.widget(0)
				importPanel.slotStartSourceImport(images)
			else:
				self.scannerLabel.setText("Error - Please try again...")
			self.state = "ready"
			self.scannerLabel.setText("Ready...")
			self.scannerButton.setPixmap(self.scanner_image)	
